using System.Diagnostics;
using System.Data;
using System.Collections;
using Microsoft.VisualBasic;
using System.Collections.Generic;
using System;

namespace Optim_RandomSearch2
{
	public delegate double MyFxDelegate(int nNumVars, ref double[] fX, ref double[] fParam);
	public delegate string SayFxDelegate();
	
	public class CRandomSearch2
	{
		
		MyFxDelegate m_MyFx;
		
		protected double MyFxEx(int N, ref double[] fX, ref double[] fParam, ref double[] fDeltaX, double fLambda)
		{
			int i;
			double[] fX2;
			
			fX2 = new double[N];
			
			for (i = 0; i < N; i++)
			{
				fX2[i] = fX[i] + fLambda * fDeltaX[i];
			}
			
            return m_MyFx(N, ref fX2, ref fParam);
		}
		
		protected bool LinSearch_DirectSearch(int N, ref double[] fX, ref double[] fParam, ref double fLambda, ref double[] fDeltaX, double fInittep, double fMinStep)
		{
			double F1, F2;
			
			object temp_object = fX;
			double[] temp_double =  (double[]) temp_object;
			object temp_object4 = fParam;
			double[] temp_double4 =  (double[]) temp_object4;
			object temp_object7 = fDeltaX;
			double[] temp_double7 =  (double[]) temp_object7;
			F1 = MyFxEx(N, ref temp_double, ref temp_double4, ref temp_double7, fLambda);
			
			do
			{
                F2 = MyFxEx(N, ref fX, ref fParam, ref fDeltaX, fLambda + fInittep);
				if (F2 < F1)
				{
					F1 = F2;
					fLambda = fLambda + fInittep;
				}
				else
				{
                    F2 = MyFxEx(N, ref  fX, ref fParam, ref fDeltaX, fLambda - fInittep);
					if (F2 < F1)
					{
						F1 = F2;
						fLambda = fLambda - fInittep;
					}
					else
					{
						// reduce search step size
						fInittep = fInittep / 10;
					}
				}
			} while (!(fInittep < fMinStep));
						
			return true;
		}
		
		public double CalcOptim(int nNumVars, ref double[] fX, ref double[] fParam, ref double[] fXLo, ref double[] fXHi, int nMaxIter, double EpsFx, ref int nIter, MyFxDelegate MyFx)
		{
			
			double[] fDeltaX;
			double fInitStep = 0.1;
			double fMinStep = 0.00001;

            double F, fBestF, fLastBestF, fLambda;
			double[] fBestX;
			int i;
            Random objRand = new Random();
			
			m_MyFx = MyFx;
			fDeltaX = new double[nNumVars];
			fBestX = new double[nNumVars];
			
			for (i = 0; i < nNumVars; i++)
			{
				fBestX[i] = fX[i];
			}
			// calculate and display function value at initial point
            fBestF = MyFx(nNumVars, ref fBestX, ref fParam);
			if (fBestF > 0)
			{
				fLastBestF = fBestF + 100;
			}
			else
			{
				fLastBestF = 100 - fBestF;
			}
			
			nIter = 0;
			do
			{
				nIter++;
				if (nIter > nMaxIter)
				{
					break;
				}
				for (i = 0; i < nNumVars; i++)
				{
					fX[i] = fXLo[i] + objRand.NextDouble() *(fXHi[i] - fXLo[i]);
					fDeltaX[i] = fX[i] - fBestX[i];
				}
				fLambda = 0.1;
                LinSearch_DirectSearch(nNumVars, ref fBestX, ref fParam, ref fLambda, ref fDeltaX, fInitStep, fMinStep);
				for (i = 0; i < nNumVars; i++)
				{
					fX[i] = fBestX[i] + fLambda * fDeltaX[i];
				}
                F = MyFx(nNumVars, ref fX, ref fParam);
				if (F < fBestF)
				{
					for (i = 0; i < nNumVars; i++)
					{
						fBestX[i] = fX[i];
					}
					fBestF = F;
					
					// test function value convergence
					if (Math.Abs(fBestF - fLastBestF) < EpsFx)
					{
						break;
					}
					fLastBestF = fBestF;
				}
			} while (true);
			
			return fBestF;
			
		}
	}
	
}
